"Functions and wrappers for creating survey structure"
from collections.abc import Sequence
from typing import Any
from pathlib import Path
from inspect import stack
from pydantic import validate_arguments
from .structure import Question, Page, Survey
from .options import QuestionOptions, PageOptions, SurveyOptions
from .utils import flatten_args


@validate_arguments
def question(
    label: str,
    question_text: str | Sequence[str],
    *answers: Any | None,
    question_type: str = "radio",
    description: str | None = None,
    options: QuestionOptions | None = None,
) -> Question | list[Question]:
    "Wrapper around Question class"
    answers_list = flatten_args(answers)
    if isinstance(question_text, str):
        return Question(
            label=label,
            question_text=question_text,
            answers=answers_list,
            question_type=question_type,
            description=description,
            options=options,
        )
    question_list = flatten_args(question_text)
    q_list = []
    for i in enumerate(question_list):
        q_list.append(
            Question(
                label=f"{label}_{i[0] + 1}",
                question_text=i[1],
                answers=answers_list,
                question_type=question_type,
                description=description,
                options=options,
            )
        )
    return q_list


def page(
    label: str,
    *questions: Question | Sequence[Question],
    title: str | None = None,
    description: str | None = None,
    options: PageOptions | None = None,
) -> Page:
    "Wrapper around Page class"
    questions_list = flatten_args(questions)
    return Page(
        label=label,
        questions=questions_list,
        title=title,
        description=description,
        options=options,
    )


def survey(
    label: str,
    *pages: Page | Sequence[Page],
    title: str | None = None,
    description: str | None = None,
    options: SurveyOptions | None = None,
    create: bool | str | Path = True,
    build: bool = True,
) -> Survey:
    "Create Survey object from pages, create json file"
    pages_list = flatten_args(pages)
    survey_obj = Survey(
        label=label,
        pages=pages_list,
        title=title,
        description=description,
        options=options,
    )
    if create and isinstance(create, bool):
        survey_obj.create(build=build)
    elif isinstance(create, str) or isinstance(create, Path):
        survey_obj.create(Path(create), build=build)
    return survey_obj


def option(type: str | None = None, **kwargs):
    "Create options object for question, page or survey"
    if type is None:
        calling_function = stack()[1].function
    else:
        calling_function = type
    if calling_function in ["question", "Question"]:
        return QuestionOptions(**kwargs)
    elif calling_function in ["page", "Page"]:
        return PageOptions(**kwargs)
    elif calling_function in ["survey", "Survey"]:
        return SurveyOptions(**kwargs)
    else:
        raise ValueError('type must be "question", "page" or "survey"')



"Wrappers for different question types."

from .tools import question
from .structure import Question, QuestionOptions


def radio(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for radio type."
    return question(
        label,
        question_text,
        *answers,
        question_type="radio",
        description=description,
        options=options
    )


def checkbox(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for checkbox type."
    return question(
        label,
        question_text,
        *answers,
        question_type="checkbox",
        description=description,
        options=options
    )


def text(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for text type."
    return question(
        label,
        question_text,
        *answers,
        question_type="text",
        description=description,
        options=options
    )


def text_long(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for text_long type."
    return question(
        label,
        question_text,
        *answers,
        question_type="text_long",
        description=description,
        options=options
    )


def dropdown(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for dropdown type."
    return question(
        label,
        question_text,
        *answers,
        question_type="dropdown",
        description=description,
        options=options
    )


def dropdown_multi(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for dropdown_multi type."
    return question(
        label,
        question_text,
        *answers,
        question_type="dropdown_multi",
        description=description,
        options=options
    )


def yes_no(
    label, question_text, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for yes_no type."
    return question(
        label,
        question_text,
        question_type="yes_no",
        description=description,
        options=options,
    )


def ranking(
    label, question_text, *answers, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for ranking type."
    return question(
        label,
        question_text,
        *answers,
        question_type="ranking",
        description=description,
        options=options
    )


def info(label: str, text: str, options=None) -> Question:
    "Wrapper around question function for info type."
    return question(label, text, question_type="info", options=options)


def slider(
    label: str,
    question_text,
    range_min=0,
    range_max=100,
    pips_values=[0, 100],
    pips_text=["0", "100"],
    description=None,
    options=None,
) -> Question | list[Question]:
    "Wrapper around question function for slider type."
    if len(pips_text) != len(pips_values):
        raise ValueError("Length of pips_text and pips_values must be equal.")
    if not options:
        options = QuestionOptions()
    else:
        options = options.copy()
    setattr(options, "range_min", range_min)
    setattr(options, "range_max", range_max)
    setattr(options, "pips_values", pips_values)
    setattr(options, "pips_text", pips_text)

    return question(
        label,
        question_text,
        question_type="slider",
        description=description,
        options=options,
    )


def matrix(
    label, columns, *rows, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for matrix type."
    return question(
        label,
        columns,
        rows,
        question_type="matrix",
        description=description,
        options=options,
    )


def matrix_dynamic(
    label, question_text, columns, description=None, options=None
) -> Question | list[Question]:
    "Wrapper around question function for matrix_dymamic type."
    return question(
        label,
        question_text,
        columns,
        question_type="matrix_dynamic",
        description=description,
        options=options,
    )



"Options objects definitions"

from pydantic import BaseModel
from collections.abc import Sequence


class QuestionOptions(BaseModel):
    "Options for Question object"
    required: bool = False
    answers_order: str = "none"
    placeholder: str | None = None
    inherit_answers: str | None = None
    inherit_answers_mode: str = "all"
    comment: bool = False
    comment_text: str = "Other"
    comment_placeholder: str = ""
    visible: bool = True
    other: bool = False
    other_text: str = "Other"
    other_placeholder: str = ""
    none: bool = False
    none_text: str = "None"
    clear_button: bool = False
    visible_if: str | None = None
    editable_if: str | None = None
    requied_if: str | None = None
    hide_number: bool = False
    range_min: int = 0
    range_max: int = 100
    pips_values: Sequence[int] = [0, 100]
    pips_text: Sequence[str] = ["0", "100"]
    allow_add_rows: bool = True
    allow_remove_rows: bool = True
    allow_rows_drag_and_drop: bool = False
    row_count: int = 1
    min_row_count: int = 0
    max_row_count: int = 1000
    add_row_text: str = "Add row"


class PageOptions(BaseModel):
    "Options for Page object"
    read_only: bool = False
    time_limit: int | None = None
    visible: bool = True
    navigation_visibility: str = "inherit"
    visible_if: str | None = None
    editable_if: str | None = None
    requied_if: str | None = None


class SurveyOptions(BaseModel):
    "Options for Survey object"
    language: str = "en"
    number_of_groups: int = 1
    timer_position: str = "none"
    timer_mode: str | None = None
    url_on_complete: str | None = None
    allow_previous: bool = True
    clear_invivsible_values: str | bool = "none"
    start_page: bool = False
